"use strict";
exports.__esModule = true;
var schematics_1 = require("@angular-devkit/schematics");
var tasks_1 = require("@angular-devkit/schematics/tasks");
var devDependencies = [
    {
        name: 'wallaby-webpack',
        version: '*'
    },
    {
        name: 'angular2-template-loader',
        version: '^0.6.2'
    }
];
function default_1(_options) {
    return function (tree, context) {
        var nx = containNxScematics(tree);
        return schematics_1.chain([
            nx
                ? schematics_1.chain([addBootstrapFileForNx(), addConfigFileForNx(tree)])
                : schematics_1.chain([
                    addConfigFile(),
                    addBootstrapFile(),
                    modifyTsconfigExcludeFile()
                ]),
            addLoadersToPackageJson(),
            addPackageInstallTask()
        ])(tree, context);
    };
}
exports["default"] = default_1;
function addConfigFile() {
    return function (tree, context) {
        var wallabyConfigSourceFileName = '/wallaby.js';
        var wallabyConfigFileName = '/wallaby.js';
        var files = schematics_1.url("../../files")(context);
        var createContent = files
            .read(wallabyConfigSourceFileName)
            .toString('utf-8');
        if (!tree.exists(wallabyConfigFileName)) {
            tree.create(wallabyConfigFileName, createContent);
        }
        return tree;
    };
}
function addBootstrapFile() {
    return function (tree, context) {
        var wallabyBootstrapSourceFileName = '/src/wallabyTest.ts';
        var wallabyBootstrapFileName = '/src/wallabyTest.ts';
        var files = schematics_1.url("../../files")(context);
        var scriptContent = files
            .read(wallabyBootstrapSourceFileName)
            .toString('utf-8');
        if (!tree.exists(wallabyBootstrapFileName)) {
            tree.create(wallabyBootstrapFileName, scriptContent);
        }
        return tree;
    };
}
function modifyTsconfigExcludeFile() {
    return function (tree) {
        if (tree.exists('/tsconfig.app.json')) {
            var tsConfigSource = tree.read('/tsconfig.app.json').toString('utf-8');
            var json = JSON.parse(tsConfigSource);
            if (json['exclude']) {
                if (!json['exclude'].includes('src/wallabyTest.ts')) {
                    json['exclude'].push('src/wallabyTest.ts');
                }
            }
            tree.overwrite('/tsconfig.app.json', JSON.stringify(json, null, 2));
        }
        else {
            throw new schematics_1.SchematicsException("Could not find tsconfig.app.json.");
        }
        return tree;
    };
}
function addLoadersToPackageJson() {
    return function (tree) {
        devDependencies.forEach(function (dependency) {
            addDependencyToPackageJson(tree, 'devDependencies', dependency.name, dependency.version);
        });
        return tree;
    };
}
function addPackageInstallTask() {
    return function (tree, context) {
        var depNames = devDependencies.map(function (d) { return d.name; }).join(' ');
        context.addTask(new tasks_1.NodePackageInstallTask({
            packageName: depNames
        }));
        return tree;
    };
}
/**
 * Adds a package to the package.json
 */
function addDependencyToPackageJson(tree, type, pkg, version) {
    if (version === void 0) { version = 'latest'; }
    if (tree.exists('/package.json')) {
        var sourceText = tree.read('/package.json').toString('utf-8');
        var json = JSON.parse(sourceText);
        if (!json[type]) {
            json[type] = {};
        }
        if (!json[type][pkg]) {
            json[type][pkg] = version;
        }
        tree.overwrite('/package.json', JSON.stringify(json, null, 2));
    }
    return tree;
}
exports.addDependencyToPackageJson = addDependencyToPackageJson;
/**
 * Work with Nx project
 */
function addConfigFileForNx(tree) {
    var wallabyConfigSourceFileName = '/nx/wallaby/wallaby.js';
    var importCompilerOptions = createCompilerOptions(getProjectLibs(tree));
    var importAlias = createAlias(tree);
    var templateSource = schematics_1.apply(schematics_1.url('../../files/nx/wallaby'), [
        schematics_1.template({
            importCompilerOptions: importCompilerOptions,
            importAlias: importAlias
        }),
        schematics_1.move('nx/wallaby/', '')
    ]);
    return schematics_1.branchAndMerge(schematics_1.mergeWith(templateSource));
}
function addBootstrapFileForNx() {
    var wallabyBootstrapSourceFileName = '/nx/wallabyTest/wallabyTest.ts';
    var templateSource = schematics_1.apply(schematics_1.url('../../files/nx/wallabyTest'), [
        schematics_1.move('nx/wallabyTest', '')
    ]);
    return schematics_1.branchAndMerge(schematics_1.mergeWith(templateSource));
}
function containNxScematics(tree) {
    var sourceText = tree.read('/package.json').toString('utf-8');
    var packageJson = JSON.parse(sourceText);
    return !!packageJson.dependencies['@nrwl/nx'];
}
function getProjectLibs(tree) {
    var sourceText = tree.read('/angular.json').toString('utf-8');
    var packageJson = JSON.parse(sourceText);
    var projects = { apps: [], libs: [] };
    for (var project in packageJson.projects) {
        if (project.includes('-e2e')) {
            continue;
        }
        var projectInfo = packageJson.projects[project];
        if (projectInfo.projectType === 'application') {
            projects.apps.push(project);
        }
        if (projectInfo.projectType === 'library') {
            projects.libs.push(project);
        }
    }
    return projects;
}
function createCompilerOptions(defaultProject) {
    var compilerOptions = [];
    compilerOptions = compilerOptions.concat(defaultProject.apps.map(function (app) {
        return "require('./apps/" + app + "/tsconfig.spec.json').compilerOptions";
    }));
    compilerOptions = compilerOptions.concat(defaultProject.libs.map(function (lib) {
        return "require('./libs/" + lib + "/tsconfig.spec.json').compilerOptions";
    }));
    return compilerOptions.join(',');
}
function createAlias(tree) {
    var sourceText = tree.read('/tsconfig.json').toString('utf-8');
    var tsconfig = JSON.parse(sourceText);
    var alias = [];
    for (var path in tsconfig.compilerOptions.paths) {
        if (!path.includes('*')) {
            var value = tsconfig.compilerOptions.paths[path];
            alias.push("'" + path + "': path.join(wallaby.projectCacheDir, '" + value + "' )");
        }
    }
    if (alias.length === 0) {
        return '';
    }
    return "alias: {\n    " + alias.join(',') + "\n  }";
}
//# sourceMappingURL=index.js.map